// pgp4pine pki.c

// See common.c for the version and license.
// Copyright (C) 1998 by Chris Wiegand

#include "includes.h"

// -----------------------------------------------------------------

int pipe_pgp(const char *cmd, FILE **in, FILE **out)
{
         int pin[2], pout[2], child_pid;

         *in = *out = NULL;

         pipe(pin);
         pipe(pout);

	sprintf(debugLine,"pipe_pgp: running: %s\n",cmd); 
	documentStatus(debugLine);

         if(!(child_pid = fork()))
         {
           // We're the child.
           close(pin[1]);
           dup2(pin[0], 0);
           close(pin[0]);

           close(pout[0]);
           dup2(pout[1], 1);
           close(pout[1]);

           execl("/bin/sh", "sh", "-c", cmd, NULL);
           _exit(127);
         }

         // Only get here if we're the parent.
         close(pout[1]);
         *out = fdopen(pout[0], "r");

         close(pin[0]);
         *in = fdopen(pin[1], "w");

         return(child_pid);
}

// ---------------------------------------------------------------------------------

void fileVerifyDecrypt(const char inFile[MAX_COMMAND_LINE_LENGTH],const char outFile[MAX_COMMAND_LINE_LENGTH])
{
	char pgpAppString[MAX_COMMAND_LINE_LENGTH];

	switch (myUserPrefs->pgpVersion) {
	case 5:
		if (myUserPrefs->addNewKeys == 0) {
			sprintf(pgpAppString,"%spgpv -K -o %s %s",myUserPrefs->pgpFolder,outFile,inFile);
		} else {
			sprintf(pgpAppString,"%spgpv -o %s %s",myUserPrefs->pgpFolder,outFile,inFile);
		}
		break;
	case 2:
		if (myUserPrefs->addNewKeys == 1) {
			// note, pgp2 doesn't support importing keys from decrypt...
			// so, we'll scan for new keys...
			sprintf(pgpAppString,"%spgp -ka %s",myUserPrefs->pgpFolder,inFile);
			runProcessFG(pgpAppString);
		}
		sprintf(pgpAppString,"%spgp %s -o %s",myUserPrefs->pgpFolder,inFile,outFile);
		break;
	case 1:
		if (myUserPrefs->addNewKeys == 1) {
			// Unsure if this applies to gpg 0.4:
			// note, gpg 0.34 doesn't support importing keys from decrypt...
			// sadly, unless the key is the FIRST thing there, it won't import it...
			// so, it's useless to try importing, at least for now...
		}
		sprintf(pgpAppString,"%sgpg --no-greeting --output %s --decrypt %s",myUserPrefs->pgpFolder,outFile,inFile);
		break;	
	}
	runProcessFG(pgpAppString);
}

// ---------------------------------------------------------------------------------

void fileSignEncrypt(const char inFileName[MAX_COMMAND_LINE_LENGTH],
		     const char outFileName[MAX_COMMAND_LINE_LENGTH],
		     const int sign, const int encrypt,
		     char recipient[EMAIL_ADDRESS_MAX_LENGTH], 
		     const int asciiArmor, const int universalText,
		     int encryptToSelf,int detachSigs)
{
	char pgpAppString[MAX_COMMAND_LINE_LENGTH];
	char pgpOptions[MAX_COMMAND_LINE_LENGTH], recipList[EMAIL_ADDRESS_MAX_LENGTH];

	strcpy(pgpOptions," ");
	strcpy(recipList," ");
	strcat(pgpOptions,myUserPrefs->extraOptions);

	if (encryptToSelf > 0) {
		if (strlen(myUserPrefs->myAddress) > 3) {
			strcat(recipient," ");
			strcat(recipient,myUserPrefs->myAddress);
			strcat(recipient," ");
		} else {
			if (myUserPrefs->pgpVersion != 5) printf("Because you didn't specify your email address in ~/.pgp4pinerc,\nI can't auto-encrypt to you as well.\n");
			encryptToSelf = -1;
		}
	}

	switch (myUserPrefs->pgpVersion) {
	case 5:
		if (detachSigs > 0) strcat(pgpOptions," -b ");
		// get recipList...
		if (encrypt > 0) {
			parseRecipientList(recipient);
			strcpy(recipList,recipient);
 			if (sign > 0) strcat(pgpOptions," -s ");
		}
		if (sign > 0 && strlen(mySecretKey) > 3) {
 				strcat(pgpOptions," -u \"");
 				strcat(pgpOptions,mySecretKey);
 				strcat(pgpOptions,"\" ");
		}
		if (asciiArmor > 0) strcat(pgpOptions," -a ");
		if (universalText > 0) strcat(pgpOptions," -t ");

		if (encrypt > 0)
			sprintf(pgpAppString,"pgpe %s %s -o %s %s",pgpOptions,recipList,outFileName,inFileName);
		else
			sprintf(pgpAppString,"pgps %s -o %s %s",pgpOptions,outFileName,inFileName);
		runProcessFG(pgpAppString);
		break;
	case 2:
		strcat(pgpOptions," -"); // we either have to add a e[s] or just s...
		if (encrypt > 0) strcat(pgpOptions,"e");
		if (sign > 0) {
			strcat(pgpOptions,"s");
			if (detachSigs > 0) strcat(pgpOptions,"b");
		}
		if (asciiArmor > 0) strcat(pgpOptions,"a");
		strcat(pgpOptions," ");
		if (encrypt > 0) strcpy(recipList,recipient);
		if (sign > 0 && strlen(mySecretKey) > 3) {
			strcat(pgpOptions," -u \"");
			strcat(pgpOptions,mySecretKey);
			strcat(pgpOptions,"\" ");
		}
		if (encrypt > 0)
			sprintf(pgpAppString,"pgp %s %s",pgpOptions,inFileName);
		else
			sprintf(pgpAppString,"pgp %s %s %s",pgpOptions,inFileName,recipList);
		runProcessFG(pgpAppString); 
		
		// now, we move the <infile>.pgp to <outfile>
		if (detachSigs > 0) {
			// we're detaching a signature, the original file isn't changed....
		} else {
			if (asciiArmor > 0)
				sprintf(pgpAppString,"mv %s.asc %s",inFileName,outFileName);
			else
				sprintf(pgpAppString,"mv %s.pgp %s",inFileName,outFileName);
			runProcessFG(pgpAppString);
		}
		break;
	case 1:
		if (detachSigs > 0) strcat(pgpOptions," -b ");
		if (encrypt > 0) {
			strcat(pgpOptions," -e ");
			parseRecipientList(recipient);
			strcpy(recipList,recipient);
			if (myUserPrefs->encryptToSelf == 1) {
				strcat(recipList," -r ");
				strcat(recipList,myUserPrefs->myAddress);
			}
		}
		if (sign > 0) {
			strcat(pgpOptions," -s ");
			if (strlen(mySecretKey) > 3) {
				strcat(pgpOptions," -u \"");
				strcat(pgpOptions,mySecretKey);
				strcat(pgpOptions,"\" ");
			}
		}
		if (myUserPrefs->asciiArmor > 0) strcat(pgpOptions, " -a ");
		if (myUserPrefs->universalText > 0) strcat(pgpOptions, " -t ");
		// if we're not encrypting, recipList is just a " ", so we're safe...
		sprintf(pgpAppString,"gpg --no-batch --comment \"Made with PGP4Pine\" --no-greeting %s %s -o %s %s",pgpOptions,recipList,outFileName,inFileName);
		runProcessFG(pgpAppString);
		break;
	}
}


void parseRecipientList(char *recipients)
{
	char tmpEmail[EMAIL_ADDRESS_MAX_LENGTH], recipList[EMAIL_ADDRESS_MAX_LENGTH];
	// recipients is passed back, so it's a pointer
	strcat(recipients," . "); // we add a space so that anything at the end is an item
	// if you don't, strtok() can segfault because there's no terminating " "...
	documentStatus("Parsing recipient list\n");

	strcpy(tmpEmail,strtok(recipients," "));
	checkRecipient(tmpEmail);
	strcpy(recipList,"-r ");
	strcat(recipList,tmpEmail);
 	do {
		strcpy(tmpEmail,strtok('\0'," "));
		if (strcmp(tmpEmail,".")) {
			checkRecipient(tmpEmail);
			strcat(recipList," -r \"");
			strcat(recipList,tmpEmail);
			strcat(recipList,"\" ");
		}
	} while(strcmp(tmpEmail,"."));
	strcpy(recipients,recipList);
	documentStatus("Recipients: ");
	documentStatus(recipients);
}

int checkRecipient(char *thisRecip)
{
	// This just checks a single recipient, when we parse the recipient list, we check each one...
	struct pkiKey *thisKey = NULL;
	struct pkiKey *lastKey = NULL;
	int found = 0, canLeave = 0;
	do {
		thisKey = publicKeyring;
		lastKey = thisKey;
		while (thisKey != NULL) {
			if (strcmp(thisRecip,thisKey->emailAddress) == 0) {
				found++;
				lastKey = thisKey;
			}
			thisKey = thisKey->nextKey;
		}
		if (found < 1) {
		    	char c;
			// okay, we didn't find them...
			printf("\n\nOops! I did not see %s in your keyring!\n",thisRecip);
			printf("Please select one of the following:\na)Open a sub-shell and add their key manually.\n");
			printf("b)Delete the recipient from the PGP list\nc)Ignore this recipient, send to them anyways.\n");
			if (myUserPrefs->pgpVersion == 5) printf("d)Try to fetch key from keyserver.\n");
			printf("Choice? ");
			scanf("%c",&c);
			printf("\n");
			switch (c) {
				case 'a':
				case 'A':
					runProcessFG("/bin/sh");
					deallocAllKeys(publicKeyring);
					getPublicKeyring(); // refresh list, try again...
					break;
				case 'b':
				case 'B':
					strcpy(thisRecip," ");
					canLeave = 1;
					break;
				case 'c':
				case 'C':
					canLeave = 1; // ignore recipient...
					break;
				case 'd':
				case 'D':
					if (myUserPrefs->pgpVersion != 5) {
						printf("Sorry, but only PGP 5 can get keys from a keyserver (for now).\n");
					} else {
						char pgpAppString[MAX_COMMAND_LINE_LENGTH];
						sprintf(pgpAppString,"pgpk -a %s",thisRecip);
						runProcessFG(pgpAppString);
						deallocAllKeys(publicKeyring);
						getPublicKeyring(); // refresh list, try again....
					}
					break;
			}
		} else if (found > 1) {
			int i=0,j=0,k=0;
			// wow..two keys...
			printf("\n\nThe following recipient has multiple keys. Please select one.\n");
			thisKey = publicKeyring; // set back to beginning
			do {
				if (strcmp(thisKey->emailAddress,thisRecip) == 0) {
					i++; // number of able and ready keys...
					printKeyInfo((char) ('a'+i-1),thisKey);
				}
				thisKey = thisKey->nextKey;
			} while (thisKey != NULL);
			j = ('a'-askAlphaRange("Please select the recipient key to use.",'a',('a'+i-1))+1);
			thisKey = publicKeyring;
			do {
				if (strcmp(thisKey->emailAddress,thisRecip) == 0) k++;
				thisKey = thisKey->nextKey;
			} while (k<j);
#ifdef USE_DISPLAY_NAME
			strcpy(thisRecip,thisKey->emailAddress);
#else
			strcpy(thisRecip,thisKey->keyID);
#endif
			canLeave = 1;
		} else {
			// only one key for this recipient
			// 0.43: chris added this because I wanted to see it...I'm going to make it a prefs thing...
			if (prefsVerboseOutput > 0) {
				printf("Key for %s found.",lastKey->displayName);
				printKeyInfo(':',lastKey);
			}
#ifdef USE_DISPLAY_NAME
			strcpy(thisRecip,lastKey->emailAddress);
#else
			strcpy(thisRecip,lastKey->keyID);
#endif
			canLeave = 1;
		}
	} while (canLeave < 1);
	return 0;
}

void printKeyInfo(char c,struct pkiKey *thisKey)
{
	printf("\n%c: %s\n\tType: %s\tBits: %s\tID: %s\n",c,thisKey->displayName,thisKey->keyType,thisKey->keySize,thisKey->keyID);
	return;
}
